@@ -8,7 +8,7 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 use Fcntl;
 use integer;
 
-$VERSION = '0.23';
+$VERSION = '0.24';
 
 require Exporter;
 require DynaLoader;
@@ -54,6 +54,13 @@ sub add_bits {
 		$data = pack("B*", $data);
 	}
 	$nbits = length($data) * 8 if $nbits > length($data) * 8;
+	if ($nbits % 8) {
+		my @c = unpack("C*", $data);
+		my $b = pop(@c);
+		$b >>= (8 - $nbits % 8);
+		push(@c, $b);
+		$data = pack("C*", @c);
+	}
 	shawrite($data, $nbits, $self);
 	return($self);
 }
@@ -457,6 +464,21 @@ So, the following two statements do the same thing:
 	$sha3->add_bits("111100001010");
 	$sha3->add_bits("\xF0\xA0", 12);
 
+Note that SHA-3 uses I<least-significant-bit ordering> for its
+internal state.  This means that
+
+	$sha3->add_bits("110");
+
+is equivalent to
+
+	$sha3->add_bits("0")->add_bits("1")->add_bits("1");
+
+The fact that SHA-2 and SHA-3 employ opposite bit-ordering schemes
+has resulted in noticeable confusion, even in the cryptographic
+community.  When programming with bitwise inputs, expect to make
+blunders, repeatedly: if the Scylla of notation doesn't devour you,
+the Charybdis of semantics almost certainly will.
+
 =item B<addfile(*FILE)>
 
 Reads from I<FILE> until EOF, and appends that data to the current
