@@ -5,8 +5,8 @@
  *
  * Copyright (C) 2012-2015 Mark Shelor, All Rights Reserved
  *
- * Version: 0.23
- * Sun Jan  4 05:36:30 MST 2015
+ * Version: 0.24
+ * Sat Jan 10 00:45:34 MST 2015
  *
  */
 
@@ -34,7 +34,7 @@ static void word2mem(UCHR *mem, W64 w)
 		*p++ = (UCHR) (w & 0xff);
 }
 
-static W64 RC[] = {	/* Keccak round constants */
+static const W64 RC[] = {	/* Keccak round constants */
 	C64(0x0000000000000001), C64(0x0000000000008082),
 	C64(0x800000000000808a), C64(0x8000000080008000),
 	C64(0x000000000000808b), C64(0x0000000080000001),
@@ -56,7 +56,7 @@ static W64 RC[] = {	/* Keccak round constants */
 static void keccak_f(W64 A[][5])
 {
 	int i;
-	W64 *rc = RC;
+	const W64 *rc = RC;
 	for (i = 0; i < 24; i++, rc++) {
 		W64 B[5][5], C[5], D[5];
 		C[0] = A[0][0]^A[0][1]^A[0][2]^A[0][3]^A[0][4];
@@ -179,12 +179,15 @@ static UCHR *digcpy(SHA3 *s)
 					break;
 				word2mem(Z, s->S[x][y]);
 			}
-		if ((outbits -= s->blocksize) > 0)
+		if ((outbits -= (int) s->blocksize) > 0)
 			keccak_f(s->S);
 	}
 	return(s->digest);
 }
 
+#define BITSET(s, pos)  s[(pos) >> 3] &  (UCHR)  (0x01 << ((pos) % 8))
+#define SETBIT(s, pos)  s[(pos) >> 3] |= (UCHR)  (0x01 << ((pos) % 8))
+#define CLRBIT(s, pos)  s[(pos) >> 3] &= (UCHR) ~(0x01 << ((pos) % 8))
 #define NBYTES(nbits)   (((nbits) + 7) >> 3)
 #define HEXLEN(nbytes)	((nbytes) << 1)
 #define B64LEN(nbytes)	(((nbytes) % 3 == 0) ? ((nbytes) / 3) * 4 \
@@ -265,43 +268,23 @@ static ULNG shabytes(UCHR *bitstr, ULNG bitcnt, SHA3 *s)
 /* shabits: updates state for bit-aligned data in s->block */
 static ULNG shabits(UCHR *bitstr, ULNG bitcnt, SHA3 *s)
 {
-	UINT i;
-	UINT gap;
-	ULNG nbits;
-	UCHR buf[1<<9];
-	UINT bufsize = sizeof(buf);
-	ULNG bufbits = (ULNG) bufsize << 3;
-	UINT nbytes = NBYTES(bitcnt);
-	ULNG savecnt = bitcnt;
+	ULNG i;
 
-	gap = 8 - s->blockcnt % 8;
-	s->block[s->blockcnt>>3] &= ~0 << gap;
-	s->block[s->blockcnt>>3] |= *bitstr >> (8 - gap);
-	s->blockcnt += bitcnt < gap ? bitcnt : gap;
-	if (bitcnt < gap)
-		return(savecnt);
-	if (s->blockcnt == s->blocksize)
-		sha3(s, s->block), s->blockcnt = 0;
-	if ((bitcnt -= gap) == 0)
-		return(savecnt);
-	while (nbytes > bufsize) {
-		for (i = 0; i < bufsize; i++)
-			buf[i] = bitstr[i] << gap | bitstr[i+1] >> (8-gap);
-		nbits = bitcnt < bufbits ? bitcnt : bufbits;
-		shabytes(buf, nbits, s);
-		bitcnt -= nbits, bitstr += bufsize, nbytes -= bufsize;
+	for (i = 0UL; i < bitcnt; i++) {
+		if (BITSET(bitstr, i))
+			SETBIT(s->block, s->blockcnt), s->blockcnt++;
+		else
+			CLRBIT(s->block, s->blockcnt), s->blockcnt++;
+		if (s->blockcnt == s->blocksize)
+			sha3(s, s->block), s->blockcnt = 0;
 	}
-	for (i = 0; i < nbytes - 1; i++)
-		buf[i] = bitstr[i] << gap | bitstr[i+1] >> (8-gap);
-	buf[nbytes-1] = bitstr[nbytes-1] << gap;
-	shabytes(buf, bitcnt, s);
-	return(savecnt);
+	return(bitcnt);
 }
 
 /* shawrite: triggers a state update using data in bitstr/bitcnt */
 static ULNG shawrite(UCHR *bitstr, ULNG bitcnt, SHA3 *s)
 {
-	if (bitcnt < 1)
+	if (!bitcnt)
 		return(0);
 	if (s->blockcnt == 0)
 		return(shadirect(bitstr, bitcnt, s));
@@ -311,8 +294,14 @@ static ULNG shawrite(UCHR *bitstr, ULNG bitcnt, SHA3 *s)
 		return(shabits(bitstr, bitcnt, s));
 }
 
-#define SETBIT(s, pos)  s[(pos) >> 3] |= (UCHR)  (0x01 << ((pos) % 8))
-#define CLRBIT(s, pos)  s[(pos) >> 3] &= (UCHR) ~(0x01 << ((pos) % 8))
+/* shapad: pads byte-aligned block with 0*1 and computes final digest */
+static void shapad(SHA3 *s)
+{
+	while (s->blockcnt < s->blocksize)
+		s->block[s->blockcnt>>3] = 0x00, s->blockcnt += 8;
+	s->block[(s->blocksize>>3)-1] |= 0x80;
+	sha3(s, s->block);
+}
 
 /* shafinish: pads remaining block(s) and computes final digest state */
 static void shafinish(SHA3 *s)
@@ -323,27 +312,15 @@ static void shafinish(SHA3 *s)
 		return;
 	s->padded = 1;
 	if (s->blockcnt % 8 == 0) {
-		s->block[s->blockcnt/8] = domain;
+		s->block[s->blockcnt>>3] = domain;
 		s->blockcnt += 8;
-		while (s->blockcnt < s->blocksize)
-			s->block[s->blockcnt/8] = 0x00, s->blockcnt += 8;
-		s->block[(s->blockcnt/8)-1] |= 0x80;
-		sha3(s, s->block);
+		shapad(s);
 		return;
 	}
-	s->block[NBYTES(s->blockcnt)-1] >>= (8 - (s->blockcnt % 8));
-	for (; domain; domain >>= 1) {
-		if (domain & 0x01)
-			SETBIT(s->block, s->blockcnt), s->blockcnt++;
-		else
-			CLRBIT(s->block, s->blockcnt), s->blockcnt++;
-		if (s->blockcnt == s->blocksize)
-			sha3(s, s->block), s->blockcnt = 0;
-	}
-	while (s->blockcnt < (s->blocksize - 1))
+	shawrite((UCHR *) &domain, s->shake ? 5 : 3, s);
+	while (s->blockcnt % 8)
 		CLRBIT(s->block, s->blockcnt), s->blockcnt++;
-	SETBIT(s->block, s->blockcnt), s->blockcnt++;
-	sha3(s, s->block);
+	shapad(s);
 }
 
 /* shasqueeze: returns pointer to squeezed digest (binary) */
@@ -359,7 +336,7 @@ static UCHR *shasqueeze(SHA3 *s)
 #define shadigest(state)	digcpy(state)
 
 /* xmap: translation map for hexadecimal encoding */
-static char xmap[] =
+static const char xmap[] =
 	"0123456789abcdef";
 
 /* shahex: returns pointer to current digest (hexadecimal) */
@@ -382,7 +359,7 @@ static char *shahex(SHA3 *s)
 }
 
 /* bmap: translation map for Base 64 encoding */
-static char bmap[] =
+static const char bmap[] =
 	"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 
 /* encbase64: encodes input (0 to 3 bytes) into Base 64 */
@@ -393,7 +370,7 @@ static void encbase64(UCHR *in, int n, char *out)
 	out[0] = '\0';
 	if (n < 1 || n > 3)
 		return;
-	Copy(in, byte, n, UCHR);
+	Copy(in, byte, (unsigned) n, UCHR);
 	out[0] = bmap[byte[0] >> 2];
 	out[1] = bmap[((byte[0] & 0x03) << 4) | (byte[1] >> 4)];
 	out[2] = bmap[((byte[1] & 0x0f) << 2) | (byte[2] >> 6)];
